package jp.ne.ruru.park.ando.memoswing.core.editors;

import java.lang.Character.UnicodeBlock;

/** rp[ */
public class MemoRubiRule implements IRule {
	/** Rg */
	public static final char CHAR_OMMENT = '';
	/** ^u */
	public static final char CHAR_TAB = '\t';
	/** pXy[X */
	public static final char CHAR_SPACE = ' ';
	/** SpXy[X */
	public static final char CHAR_D_SPACE = '@';
	/** pCv */
	public static final char CHAR_PIPE = '|';
	/** SppCv */
	public static final char CHAR_D_PIPE = 'b';
	/** r */
	public static final char CHAR_RUBI1_L = '(';
	/** rSp */
	public static final char CHAR_D_RUBI1_L = 'i';
	/** Er */
	public static final char CHAR_RUBI1_R = ')';
	/** ErSp */
	public static final char CHAR_D_RUBI1_R = 'j';
	/** r */
	public static final char CHAR_RUBI2_L = 's';
	/** Er */
	public static final char CHAR_RUBI2_R = 't';
	/** ő僋r */
    private static final int COUNT_MAX= 10;

    /** rpg[N */
    private final IToken rubiToken;
    /**  */
    private enum STATUS {
    	/** G[ */
    	ERROR,
    	/** Jn */
    	START,
    	/** r */
    	BASE,
    	/** Rg */
    	COMMENT_KANJI,
    	/** ^O */
    	LEFT_TAG,
    	/** r */
    	RUBI1,
    	/** r */
    	RUBI2,
    	/** I */
    	END};
    /**
     * RXgN^
     * @param rubiToken rpg[N
     */
    public MemoRubiRule(IToken rubiToken) {
        Assert.isNotNull(rubiToken);

        this.rubiToken = rubiToken;
    }
    /*
     * @see IRule#evaluate(ICharacterScanner)
     */
    @Override
    public IToken evaluate(ICharacterScanner scanner) {
        //
        STATUS status = STATUS.START;
        //
        int count = 0;
        int countBase = 0;
        int countRubi = 0;
        while (true) {
            int c = scanner.read();
            count++;
            if ((c == ICharacterScanner.EOF)
            		|| (c == '\r')
            		|| (c == '\n')) {
                break;
            }
            switch (status) {
            case START:
                if ((c == CHAR_PIPE) || (c == CHAR_D_PIPE)) {
                    status = STATUS.BASE;
                    break;
                } else if (UnicodeBlock.of( c ) == UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS) {
                    status = STATUS.COMMENT_KANJI;
                    countBase++;
                    break;
                }
                status = STATUS.ERROR;
                break;
            case BASE:
                if ((c == CHAR_RUBI1_L) || (c == CHAR_D_RUBI1_L)) {
                    status = STATUS.RUBI1;
                    break;
                }
                if (c == CHAR_RUBI2_L) {
                    status = STATUS.RUBI2;
                    break;
                }
                countBase++;
                if (COUNT_MAX < countBase) {
                	status = STATUS.ERROR;
                }
                break;
            case COMMENT_KANJI:
                if ((c == CHAR_RUBI1_L) || (c == CHAR_D_RUBI1_L)) {
                    status = STATUS.RUBI1;
                    break;
                }
                if (c == CHAR_RUBI2_L) {
                    status = STATUS.RUBI2;
                    break;
                }
                if ((UnicodeBlock.of( c ) == UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS)) {
                    countBase++;
                    if (COUNT_MAX < countBase) {
                        status = STATUS.ERROR;
                    }
                    break;
                }
                status = STATUS.ERROR;
                break;    
            case RUBI1:
                if ((c == CHAR_RUBI1_R) || (c == CHAR_D_RUBI1_R)
                		|| (c == CHAR_RUBI2_R)) {
                    status = STATUS.END;
                    break;
                }
                if (c == CHAR_RUBI2_R) {
                    status = STATUS.ERROR;
                    break;
                }
                if ((UnicodeBlock.of( c ) == UnicodeBlock.KATAKANA)
                        || (UnicodeBlock.of( c ) == UnicodeBlock.HIRAGANA)
                        || (c == 'E')
                        || (c == '[')
                        || (c == 'c')
                		|| (c == CHAR_TAB)
                		|| (c == CHAR_SPACE)
                		|| (c == CHAR_D_SPACE)) {
                    countRubi++;
                    if (COUNT_MAX < countRubi) {
                        status = STATUS.ERROR;
                    }
                    break;
                }
                status = STATUS.ERROR;
                break;
            case RUBI2:
                if ((c == CHAR_RUBI1_R) || (c == CHAR_D_RUBI1_R)) {
                    status = STATUS.ERROR;
                    break;
                }
                if (c == CHAR_RUBI2_R) {
                    status = STATUS.END;
                    break;
                }
                countRubi++;
                if (COUNT_MAX < countRubi) {
                	status = STATUS.ERROR;
                }
                break;
            default:
                status = STATUS.ERROR;
                break;
            }
            if ((status == STATUS.ERROR) || (status == STATUS.END)) {
                break;
            }
        }
        if ((countBase == 0) || (countRubi == 0)) {
            status = STATUS.ERROR;
        }
        if (status == STATUS.END) {
        	return rubiToken;
        }
        for (int i = 0 ; i < count ; i++) {
        	scanner.unread();
        }
        return Token.UNDEFINED;
    }
}
